@@ -2,9 +2,9 @@ package Net::IPAM::IP;
 
 use strict;
 use warnings;
-our $VERSION = '1.12';
+our $VERSION = '1.13';
 
-use Socket qw/AF_INET AF_INET6/;
+use Socket qw/:addrinfo AF_INET AF_INET6 AF_UNSPEC SOCK_RAW/;
 
 # On some platforms, inet_pton accepts various forms of invalid input or discards valid input.
 # In this case use a (slower) pure-perl implementation for Socket::inet_pton.
@@ -83,7 +83,7 @@ Returns undef on illegal input.
 
 sub new {
   croak 'wrong method call' unless defined $_[1];
-  my $self = bless( {}, ref $_[0] || $_[0]);
+  my $self = bless( {}, ref $_[0] || $_[0] );
 
   # IPv4
   if ( index( $_[1], ':' ) < 0 ) {
@@ -281,7 +281,7 @@ Returns the next IP address, returns undef on overflow.
 
 sub incr {
   my $n = incr_n( $_[0]->bytes ) // return;
-  return (ref $_[0])->new_from_bytes($n);
+  return ( ref $_[0] )->new_from_bytes($n);
 }
 
 =head2 expand
@@ -349,6 +349,116 @@ sub reverse {
   die 'logic error,';
 }
 
+=head2 getname([$error_cb])
+
+Returns the DNS name for the ip object or undef if there is no PTR RR.
+
+  say Net::IPAM::IP->new('2001:4860:4860::8888')->getname;   # dns.google.
+
+L</"getname"> calls the L<Socket> functions C<< getaddrinfo() >> and C<< getnameinfo() >> under the hood.
+
+With no error callback L</getname> just calls C<< warn() >> on error.
+
+=head3 LIMITATION:
+
+Returns just one name even if the IP has more than one PTR RR. This is a limitation
+of Socket::getnameinfo. If you need all names for IPs with more than one PTR RR then you should
+use L<Net::DNS> or similar modules.
+
+=cut
+
+sub getname {
+  my $error_cb = $_[1] // sub { warn "@_" };
+
+  my ( $err, @res ) =
+    getaddrinfo( $_[0], '', { socktype => SOCK_RAW, flags => AI_NUMERICHOST, family => AF_UNSPEC } );
+
+  if ($err) {
+    $error_cb->( $err, "getaddrinfo($_[0])" );
+    return;
+  }
+
+  my $name;
+  ( $err, $name ) = getnameinfo( $res[0]->{addr}, NI_NAMEREQD, NIx_NOSERV );
+  if ($err) {
+
+    # no error, just no resolveable name
+    return if $err == EAI_NONAME;
+
+    $error_cb->( $err, "getnameinfo($_[0])" );
+    return;
+  }
+
+  return $name;
+}
+
+=head1 CLASS-METHODS
+
+L<Net::IPAM::IP> implements the following class methods:
+
+=head2 getaddrs($name, [$error_cb])
+
+Returns a list of objects for a given $name or undef if there is no B<A> or B<AAAA> RR record.
+
+  my @ips = Net::IPAM::IP->getaddrs('dns.google.');
+  say "@ips";  #  8.8.8.8 8.8.4.4 2001:4860:4860::8844 2001:4860:4860::8888
+
+L</"getaddrs"> calls the L<Socket> functions C<< getaddrinfo() >> and C<< getnameinfo() >> under the hood.
+
+With no error callback L</getaddrs> just calls C<< warn() >> on error.
+
+For granular error handling use your own error callback:
+
+  my $my_error_cb = sub {
+    my ( $error, $msg) = @_;
+    # check the $error and do what you want with error and message
+    ...
+  }
+
+  my @ips = Net::IPAM::IP->getaddrs( $name, $my_error_cb );
+
+or shut up the default error handler with:
+
+  my @ips = Net::IPAM::IP->getaddrs( $name, sub { } );
+
+=cut
+
+sub getaddrs {
+  my ( $class, $name, $error_cb ) = @_;
+  $error_cb //= sub { warn "@_" };
+
+  unless ( defined $name ) {
+    $error_cb->( 0, "missing argument" );
+    return;
+  }
+
+  my ( $err, @res ) = getaddrinfo( $name, "", { socktype => SOCK_RAW, family => AF_UNSPEC } );
+
+  if ($err) {
+
+    # no error, just no resolveable name
+    return if $err == EAI_NONAME;
+
+    $error_cb->( $err, "getaddrinfo($name)" );
+    return;
+  }
+
+  my @ips;
+
+  while ( my $ai = shift @res ) {
+    my ( $err, $ip ) = getnameinfo( $ai->{addr}, NI_NUMERICHOST, NIx_NOSERV );
+
+    if ($err) {
+      $error_cb->( $err, "getnameinfo($name)" );
+      return;
+    }
+
+    push @ips, $class->new($ip);
+  }
+
+  return @ips;
+}
+
 =head1 FUNCTIONS
 
 L<Net::IPAM::IP> implements the following functions;
